/*
 * GSI Device Library for Host code running User Tasks remotely on GSI's APU System
 *
 * Copyright (C) 2019, GSI Technology, Inc. All rights reserved.
 *
 * This software source code is the sole property of GSI Technology, Inc.
 * and is proprietary and confidential.
 */

#ifndef GSI_APL_DEFS_H_
#define GSI_APL_DEFS_H_

#include <stdint.h>

#define _GSI_BUILD_ASSERT(e)	((void)sizeof(char [!!(e) - 1]))

#define GSI_BUILD_ASSERT_SIZE(struct_class, type, size) \
static inline void gsi_check_##type(void) \
{ \
	_GSI_BUILD_ASSERT(sizeof(struct_class type) == (size)); \
}

#define GSI_BUILD_ASSERT_CONST(name, expr) \
static inline void gsi_check_const_##name (void) \
{ \
	_GSI_BUILD_ASSERT(expr); \
}

typedef unsigned int uint;
typedef uint8_t u8;
typedef uint16_t u16;
typedef uint32_t u32;
typedef uint64_t u64;


enum section_bits {
	SB_0 = 0,
	SB_1 = 1,
	SB_2 = 2,
	SB_3 = 3,
	SB_4 = 4,
	SB_5 = 5,
	SB_6 = 6,
	SB_7 = 7,
	SB_8 = 8,
	SB_9 = 9,
	SB_10 = 10,
	SB_11 = 11,
	SB_12 = 12,
	SB_13 = 13,
	SB_14 = 14,
	SB_15 = 15,
	SB_16 = 16,
	SB_17 = 17,
	SB_18 = 18,
	SB_19 = 19,
	SB_20 = 20,
	SB_21 = 21,
	SB_22 = 22,
	SB_23 = 23,

	NUM_SBS = 16,
};

enum sm_bits {
	SM_BIT_0 = 0,
	SM_BIT_1 = 1,
	SM_BIT_2 = 2,
	SM_BIT_3 = 3,
	SM_BIT_4 = 4,
	SM_BIT_5 = 5,
	SM_BIT_6 = 6,
	SM_BIT_7 = 7,
	SM_BIT_8 = 8,
	SM_BIT_9 = 9,
	SM_BIT_10 = 10,
	SM_BIT_11 = 11,
	SM_BIT_12 = 12,
	SM_BIT_13 = 13,
	SM_BIT_14 = 14,
	SM_BIT_15 = 15,
};

enum gsi_apu_defs {
	GSI_APU_NUM_APUCS = 4,
	GSI_APUC_NUM_APCS = 2,
};

enum gsi_apc_defs {
	GSI_APC_NUM_COLS = 4096,
	GSI_APC_NUM_HALF_BANKS = 2,				/* number of half banks in a bank */
	GSI_APC_NUM_HALF_BANK_COLS = GSI_APC_NUM_COLS / GSI_APC_NUM_HALF_BANKS,
////
////	/* RSP timing */
////	GSI_APC_RSP_PROP_WAIT = 1,
////	GSI_APC_RSP_WAIT = 3,
////	GSI_APC_RSP_END_WAIT_32K = 1,
////	GSI_APC_RSP_END_WAIT_2K = 3,
////	GSI_APC_RSP_END_WAIT_RET = 1,
////	GSI_APC_RSP_RET_WAIT = 1,
////
////	GSI_APC_DEF_VD16_WAIT = 2,
////	GSI_APC_DEF_VD64_WAIT = 3,
////
////	GSI_APC_L2ARB_WAIT = 3,					/* Delay of L2 operation after L2.start */
////
// 	GSI_APUC_IMEM_SIZE = 128 * 1024,
////
	GSI_APC_CBUS_BITS = 64,

	GSI_APC_PARITY_SET_BITS = 3,
	GSI_APC_PARITY_SET = 1 << GSI_APC_PARITY_SET_BITS,	/* 1 parity bit per 8 data bits */
	GSI_APC_PARITY_SET_MASK = GSI_APC_PARITY_SET - 1,
};


enum gsi_mmb_defs {
	/* size parameters */
	GSI_MMB_NUM_BANKS = 4,		/* number of banks in a APC */
////
////	/* global controls */
////	GSI_MMB_CTL_GGRVD16_BIT = 0,	/* global rvd16 precharge control */
////	GSI_MMB_CTL_GGRVD4_BIT,		/* global rvd4 precharge control */
////
////	GSI_MMB_CTL_BANK_EN_BIT,	/* bank_en(7:0) */
////	GSI_MMB_CTL_BANK_EN_BITS = GSI_APUC_NUM_APCS * GSI_MMB_NUM_BANKS,
////
////	GSI_MMB_CTL_RSP_BIT = GSI_MMB_CTL_BANK_EN_BIT + GSI_MMB_CTL_BANK_EN_BITS,
////	GSI_MMB_CTL_RSPSTARTRET_BIT =	/* begins inbound rsp functionality */
////	        GSI_MMB_CTL_RSP_BIT,
////	GSI_MMB_CTL_RSPEND_BIT,		/* end inbount and outbound rsp functionality */
////	GSI_MMB_CTL_RSP32K_BIT,		/* outbound rsp functionality to rsp32K.bus */
////	GSI_MMB_CTL_RSP2K_BIT,		/* outbound rsp functionality to rsp2K.bus */
////	GSI_MMB_CTL_RSP256_BIT,		/* outbound rsp functionality to rsp256.bus */
////	GSI_MMB_NUM_CTL,
////
////	GSI_MMB_CTL_RSP_BITS = GSI_MMB_NUM_CTL - GSI_MMB_CTL_RSP_BIT,
////
////	GSI_MMB_RSP2K_NUM_COLS = GSI_APC_NUM_HALF_BANK_COLS,
////	GSI_MMB_RSP16_NUM_PER_2K = _DIV_ROUND_UP_NUM(GSI_MMB_RSP2K_NUM_COLS, 16),
////	GSI_MMB_RSP256_NUM_PER_2K = _DIV_ROUND_UP_NUM(GSI_MMB_RSP2K_NUM_COLS, 256),
};

enum l1_addr_regs {
	L1_ADDR_REG_0 = 0,
	L1_ADDR_REG_1 = 1,
	L1_ADDR_REG_2 = 2,
	L1_ADDR_REG_3 = 3,

	NUM_L1_ADDR_REGS,
};

enum gsi_blsect_defs {
	GSI_BLSECT_NUM_ROWS = 24,
	GSI_BLSECT_RE_BITS = GSI_BLSECT_NUM_ROWS,
	/* Encoded WE control lines */
	GSI_BLSECT_EWE_MASK_SHIFT_BITS = 3,
	GSI_BLSECT_EWE_GRP_BITS = 2,
	GSI_BLSECT_EWE_MASK_BITS = 1 << GSI_BLSECT_EWE_MASK_SHIFT_BITS,
	GSI_BLSECT_EWE_BITS = GSI_BLSECT_EWE_GRP_BITS + GSI_BLSECT_EWE_MASK_BITS,

	GSI_BLSECT_WMUX_BITS = 3,
};

enum gsi_spreader_defs {
	GSI_SPREADER_NUM_LANES = 4,
};

enum gsi_l1_defs {
	GSI_L1_NUM_BANKS = 4,
	GSI_L1_NUM_GRPS = 4,
	GSI_L1_NUM_DATA_ROWS_PER_GRP = 192,
	GSI_L1_NUM_PARITY_ROWS_PER_GRP = GSI_L1_NUM_DATA_ROWS_PER_GRP / GSI_APC_PARITY_SET,
	GSI_L1_NUM_PHYS_ROWS_PER_GRP = GSI_L1_NUM_DATA_ROWS_PER_GRP + GSI_L1_NUM_PARITY_ROWS_PER_GRP,
	GSI_L1_CTL_ROW_IN_SET_ADDR_BITS = 4,	/* Select 1 of 9 rows to be read or written in the selected L1 Byte */
	GSI_L1_CTL_SET_ADDR_BITS = 5,	/* Select 1 of 24 sets of 9-row L1 Bytes */
	GSI_L1_CTL_ROW_ADDR_BITS = GSI_L1_CTL_ROW_IN_SET_ADDR_BITS + GSI_L1_CTL_SET_ADDR_BITS,
	GSI_L1_CTL_GRP_ADDR_BITS = 2,	/* Select 1 of 4 L1 Groups in the selected L1 Bank */
	GSI_L1_CTL_BANK_ADDR_BITS = 2,	/* Select 1 of 4 L1 Banks */
	GSI_L1_L2_CTL_L1_ADDR_BITS = GSI_L1_CTL_ROW_ADDR_BITS + GSI_L1_CTL_GRP_ADDR_BITS + GSI_L1_CTL_BANK_ADDR_BITS,
	GSI_L1_VA_SET_DATA_ADDR_BITS = GSI_APC_PARITY_SET_BITS,
	GSI_L1_VA_SET_DATA_ROWS = 1 << GSI_L1_VA_SET_DATA_ADDR_BITS,
	GSI_L1_VA_SET_PARITY_ADDR_BITS = 1,
	GSI_L1_VA_SET_PARITY_ROWS = 1,
	GSI_L1_VA_SET_ADDR_BITS = GSI_L1_VA_SET_DATA_ADDR_BITS + GSI_L1_VA_SET_PARITY_ADDR_BITS,
	GSI_L1_VA_SET_ADDR_ROWS = 1 << GSI_L1_VA_SET_ADDR_BITS,
	GSI_L1_VA_NUM_SETS = GSI_L1_NUM_DATA_ROWS_PER_GRP / (1 << GSI_L1_VA_SET_DATA_ADDR_BITS),
	GSI_L1_VA_NUM_ROWS = GSI_L1_VA_SET_ADDR_ROWS * GSI_L1_VA_NUM_SETS,
};

enum gsi_l2_defs {
	GSI_L2_CTL_ROW_ADDR_BIT_IDX_BITS = 4,
	GSI_L2_CTL_ROW_ADDR_BIT_IDX_MASK = (1 << GSI_L2_CTL_ROW_ADDR_BIT_IDX_BITS) - 1,
	GSI_L2_CTL_ROW_ADDR_BYTE_IDX_BITS = 3,
	GSI_L2_CTL_ROW_ADDR_BYTE_IDX_MASK = (1 << GSI_L2_CTL_ROW_ADDR_BYTE_IDX_BITS) - 1,
	GSI_L2_CTL_ROW_ADDR_BITS = GSI_L2_CTL_ROW_ADDR_BIT_IDX_BITS + GSI_L2_CTL_ROW_ADDR_BYTE_IDX_BITS,
	GSI_L2_CTL_ROW_ADDR_MASK = (1 << GSI_L2_CTL_ROW_ADDR_BITS) - 1,
	GSI_L1_L2_CTL_L2_ADDR_BITS = GSI_L2_CTL_ROW_ADDR_BITS,
};

enum valu_consts {
	VALU_INST_TYPE_BITS = 3,

	VALU_SMAP_REGS_BITS	= 4,
	VALU_SMAP_NUM_REGS	= 1 << VALU_SMAP_REGS_BITS,
	VALU_SMAP_SHIFT_BITS	= 4,

	VALU_RDLOGIC_REGS_BITS	= 4,
	VALU_RDLOGIC_NUM_REGS	= 1 << VALU_RDLOGIC_REGS_BITS,

	VALU_RDLOGIC_REG_RMUX0_BIT = 0,
	VALU_RDLOGIC_REG_RMUX1_BIT,
	VALU_RDLOGIC_REG_RACCOR_BIT,
	VALU_RDLOGIC_REG_RACCAND_BIT,
	VALU_RDLOGIC_REG_RACCXOR_BIT,
	VALU_RDLOGIC_REG_RWINHSET_BIT,
	VALU_RDLOGIC_REG_RWINHRST_BIT,
	VALU_RDLOGIC_REG_WIDTH,

	VALU_WMUX_CTL_BITS	= GSI_BLSECT_WMUX_BITS,

	VALU_ROWNUM_REGS_BITS	= 4,
	VALU_ROWNUM_NUM_REGS	= 1 << VALU_ROWNUM_REGS_BITS,
	VALU_ROWNUM_REG_WIDTH	= 5,

	VALU_ROWMASK_REGS_BITS	= 2,
	VALU_ROWMASK_NUM_REGS	= 1 << VALU_ROWMASK_REGS_BITS,
	VALU_ROWMASK_SHIFT_BITS	= 4,
	VALU_L1_ADDR_REGS_BITS	= 2,
	VALU_L1_ADDR_NUM_REGS	= 1 << VALU_L1_ADDR_REGS_BITS,
	VALU_L1_ADDR_REG_WIDTH = GSI_L1_L2_CTL_L1_ADDR_BITS,
	VALU_RE_REG_WIDTH	= GSI_BLSECT_RE_BITS,
	VALU_EWE_REG_WIDTH	= GSI_BLSECT_EWE_BITS,

	VALU_L2_ADDR_REGS_BITS	= 0,
	VALU_L2_ADDR_NUM_REGS	= 1 << VALU_L2_ADDR_REGS_BITS,
	VALU_L2_ADDR_REG_WIDTH = GSI_L1_L2_CTL_L2_ADDR_BITS,
};

/* Conforms to APU Arch Spec r0.92 */
enum gsi_apuc_ext_reg_id {
	/* 0x00-0x0F RowNum */
	GSI_APUC_ER_ROWNUM_0	= 0x00,
	GSI_APUC_ER_ROWNUM_1,
	GSI_APUC_ER_ROWNUM_2,
	GSI_APUC_ER_ROWNUM_3,
	GSI_APUC_ER_ROWNUM_4,
	GSI_APUC_ER_ROWNUM_5,
	GSI_APUC_ER_ROWNUM_6,
	GSI_APUC_ER_ROWNUM_7,
	GSI_APUC_ER_ROWNUM_8,
	GSI_APUC_ER_ROWNUM_9,
	GSI_APUC_ER_ROWNUM_10,
	GSI_APUC_ER_ROWNUM_11,
	GSI_APUC_ER_ROWNUM_12,
	GSI_APUC_ER_ROWNUM_13,
	GSI_APUC_ER_ROWNUM_14,
	GSI_APUC_ER_ROWNUM_15,

	/* 0x10-0x1F Rdlogic */
	GSI_APUC_ER_RDLOGIC_0	= 0x10,
	GSI_APUC_ER_RDLOGIC_1,
	GSI_APUC_ER_RDLOGIC_2,
	GSI_APUC_ER_RDLOGIC_3,
	GSI_APUC_ER_RDLOGIC_4,
	GSI_APUC_ER_RDLOGIC_5,
	GSI_APUC_ER_RDLOGIC_6,
	GSI_APUC_ER_RDLOGIC_7,
	GSI_APUC_ER_RDLOGIC_8,
	GSI_APUC_ER_RDLOGIC_9,
	GSI_APUC_ER_RDLOGIC_10,
	GSI_APUC_ER_RDLOGIC_11,
	GSI_APUC_ER_RDLOGIC_12,
	GSI_APUC_ER_RDLOGIC_13,
	GSI_APUC_ER_RDLOGIC_14,
	GSI_APUC_ER_RDLOGIC_15,

	/* 0x20-0x2F Smap */
	GSI_APUC_ER_SMAP_0	= 0x20,
	GSI_APUC_ER_SMAP_1,
	GSI_APUC_ER_SMAP_2,
	GSI_APUC_ER_SMAP_3,
	GSI_APUC_ER_SMAP_4,
	GSI_APUC_ER_SMAP_5,
	GSI_APUC_ER_SMAP_6,
	GSI_APUC_ER_SMAP_7,
	GSI_APUC_ER_SMAP_8,
	GSI_APUC_ER_SMAP_9,
	GSI_APUC_ER_SMAP_10,
	GSI_APUC_ER_SMAP_11,
	GSI_APUC_ER_SMAP_12,
	GSI_APUC_ER_SMAP_13,
	GSI_APUC_ER_SMAP_14,
	GSI_APUC_ER_SMAP_15,

	/* 0x30-0x33 RE_mask */
	GSI_APUC_ER_RE_MASK_0	= 0x30,
	GSI_APUC_ER_RE_MASK_1,
	GSI_APUC_ER_RE_MASK_2,
	GSI_APUC_ER_RE_MASK_3,

	/* 0x34-0x37 eWE_mask */
	GSI_APUC_ER_EWE_MASK_0	= 0x34,
	GSI_APUC_ER_EWE_MASK_1,
	GSI_APUC_ER_EWE_MASK_2,
	GSI_APUC_ER_EWE_MASK_3,

	/* 0x38-0x3B L1_Address */
	GSI_APUC_ER_L1_ADDR_0	= 0x38,
	GSI_APUC_ER_L1_ADDR_1,
	GSI_APUC_ER_L1_ADDR_2,
	GSI_APUC_ER_L1_ADDR_3,

	/* 0x3C L2_Address */
	GSI_APUC_ER_L2_ADDR_0	= 0x3C,

	/* 0x3D BankEn */
	GSI_APUC_ER_BANK_EN_0	= 0x3D,

	/* 0x3E IFIN */
	GSI_APUC_ER_IFIN	= 0x3E,

	GSI_APUC_ER_NUM_REGS
};

#endif /* GSI_APL_DEFS_H_ */
