/*
 * Libc emulation for APC build
 */

#ifndef APC_LIBC_H
#define APC_LIBC_H

#include <gsi/libsys/config.h>

#ifndef GSI_LIBSYS_NOSTDLIB
//#error This header must only be included when building for APUC
#elif !defined(GSI_LIBSYS_X86)

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>

#ifndef ETIME
#define ETIME 62
#endif

/*
 * types
 */

#ifndef INT_MAX
#  define INT_MAX       2147483647
#  define UINT_MAX	4294967295U
#  define INT_MIN       (-INT_MAX - 1)

#  if __WORDSIZE == 64
#   define LONG_MAX     9223372036854775807L
#   define ULONG_MAX    18446744073709551615UL
#  else
#   define LONG_MAX     2147483647L
#   define ULONG_MAX    4294967295UL
#  endif
#  define LONG_MIN      (-LONG_MAX - 1L)
#endif
#else

#include <gsi/libsys/types.h>

/*
 * types
 */

#ifndef INT_MAX
#  define INT_MAX       2147483647
#  define UINT_MAX	4294967295U
#  define INT_MIN       (-INT_MAX - 1)

#  if __WORDSIZE == 64
#   define LONG_MAX     9223372036854775807L
#   define ULONG_MAX    18446744073709551615UL
#  else
#   define LONG_MAX     2147483647L
#   define ULONG_MAX    4294967295UL
#  endif
#  define LONG_MIN      (-LONG_MAX - 1L)
#endif

/*
 * error codes
 */

//extern int errno;

#ifndef EOF
#define EOF -1
#endif

#ifndef EPERM
#define EPERM            1      /* Operation not permitted */
#define ENOENT           2      /* No such file or directory */
#define ESRCH            3      /* No such process */
#define EINTR            4      /* Interrupted system call */
#define EIO              5      /* I/O error */
#define ENXIO            6      /* No such device or address */
#define E2BIG            7      /* Argument list too long */
#define ENOEXEC          8      /* Exec format error */
#define EBADF            9      /* Bad file number */
#define ECHILD          10      /* No child processes */
#define EAGAIN          11      /* Try again */
#define ENOMEM          12      /* Out of memory */
#define EACCES          13      /* Permission denied */
#define EFAULT          14      /* Bad address */
#define ENOTBLK         15      /* Block device required */
#define EBUSY           16      /* Device or resource busy */
#define EEXIST          17      /* File exists */
#define EXDEV           18      /* Cross-device link */
#define ENODEV          19      /* No such device */
#define ENOTDIR         20      /* Not a directory */
#define EISDIR          21      /* Is a directory */
#define EINVAL          22      /* Invalid argument */
#define ENFILE          23      /* File table overflow */
#define EMFILE          24      /* Too many open files */
#define ENOTTY          25      /* Not a typewriter */
#define ETXTBSY         26      /* Text file busy */
#define EFBIG           27      /* File too large */
#define ENOSPC          28      /* No space left on device */
#define ESPIPE          29      /* Illegal seek */
#define EROFS           30      /* Read-only file system */
#define EMLINK          31      /* Too many links */
#define EPIPE           32      /* Broken pipe */
#define EDOM            33      /* Math argument out of domain of func */
#define ERANGE          34      /* Math result not representable */

#define EDEADLK         35      /* Resource deadlock would occur */
#define ENAMETOOLONG    36      /* File name too long */
#define ENOLCK          37      /* No record locks available */

#define ENOSYS          38      /* Invalid system call number */

#define ENOTEMPTY       39      /* Directory not empty */
#define ELOOP           40      /* Too many symbolic links encountered */
#define EWOULDBLOCK     EAGAIN  /* Operation would block */
#define ENOMSG          42      /* No message of desired type */
#define EIDRM           43      /* Identifier removed */
#define ECHRNG          44      /* Channel number out of range */
#define EL2NSYNC        45      /* Level 2 not synchronized */
#define EL3HLT          46      /* Level 3 halted */
#define EL3RST          47      /* Level 3 reset */
#define ELNRNG          48      /* Link number out of range */
#define EUNATCH         49      /* Protocol driver not attached */
#define ENOCSI          50      /* No CSI structure available */
#define EL2HLT          51      /* Level 2 halted */
#define EBADE           52      /* Invalid exchange */
#define EBADR           53      /* Invalid request descriptor */
#define EXFULL          54      /* Exchange full */
#define ENOANO          55      /* No anode */
#define EBADRQC         56      /* Invalid request code */
#define EBADSLT         57      /* Invalid slot */

#define EDEADLOCK       EDEADLK

#define EBFONT          59      /* Bad font file format */
#define ENOSTR          60      /* Device not a stream */
#define ENODATA         61      /* No data available */
#define ETIME           62      /* Timer expired */
#define ENOSR           63      /* Out of streams resources */
#define ENONET          64      /* Machine is not on the network */
#define ENOPKG          65      /* Package not installed */
#define EREMOTE         66      /* Object is remote */
#define ENOLINK         67      /* Link has been severed */
#define EADV            68      /* Advertise error */
#define ESRMNT          69      /* Srmount error */
#define ECOMM           70      /* Communication error on send */
#define EPROTO          71      /* Protocol error */
#define EMULTIHOP       72      /* Multihop attempted */
#define EDOTDOT         73      /* RFS specific error */
#define EBADMSG         74      /* Not a data message */
#define EOVERFLOW       75      /* Value too large for defined data type */
#define ENOTUNIQ        76      /* Name not unique on network */
#define EBADFD          77      /* File descriptor in bad state */
#define EREMCHG         78      /* Remote address changed */
#define ELIBACC         79      /* Can not access a needed shared library */
#define ELIBBAD         80      /* Accessing a corrupted shared library */
#define ELIBSCN         81      /* .lib section in a.out corrupted */
#define ELIBMAX         82      /* Attempting to link in too many shared libraries */
#define ELIBEXEC        83      /* Cannot exec a shared library directly */
#define EILSEQ          84      /* Illegal byte sequence */
#define ERESTART        85      /* Interrupted system call should be restarted */
#define ESTRPIPE        86      /* Streams pipe error */
#define EUSERS          87      /* Too many users */
#define ENOTSOCK        88      /* Socket operation on non-socket */
#define EDESTADDRREQ    89      /* Destination address required */
#define EMSGSIZE        90      /* Message too long */
#define EPROTOTYPE      91      /* Protocol wrong type for socket */
#define ENOPROTOOPT     92      /* Protocol not available */
#define EPROTONOSUPPORT 93      /* Protocol not supported */
#define ESOCKTNOSUPPORT 94      /* Socket type not supported */
#define EOPNOTSUPP      95      /* Operation not supported on transport endpoint */
#define EPFNOSUPPORT    96      /* Protocol family not supported */
#define EAFNOSUPPORT    97      /* Address family not supported by protocol */
#define EADDRINUSE      98      /* Address already in use */
#define EADDRNOTAVAIL   99      /* Cannot assign requested address */
#define ENETDOWN        100     /* Network is down */
#define ENETUNREACH     101     /* Network is unreachable */
#define ENETRESET       102     /* Network dropped connection because of reset */
#define ECONNABORTED    103     /* Software caused connection abort */
#define ECONNRESET      104     /* Connection reset by peer */
#define ENOBUFS         105     /* No buffer space available */
#define EISCONN         106     /* Transport endpoint is already connected */
#define ENOTCONN        107     /* Transport endpoint is not connected */
#define ESHUTDOWN       108     /* Cannot send after transport endpoint shutdown */
#define ETOOMANYREFS    109     /* Too many references: cannot splice */
#define ETIMEDOUT       110     /* Connection timed out */
#define ECONNREFUSED    111     /* Connection refused */
#define EHOSTDOWN       112     /* Host is down */
#define EHOSTUNREACH    113     /* No route to host */
#define EALREADY        114     /* Operation already in progress */
#define EINPROGRESS     115     /* Operation now in progress */
#define ESTALE          116     /* Stale file handle */
#define EUCLEAN         117     /* Structure needs cleaning */
#define ENOTNAM         118     /* Not a XENIX named type file */
#define ENAVAIL         119     /* No XENIX semaphores available */
#define EISNAM          120     /* Is a named type file */
#define EREMOTEIO       121     /* Remote I/O error */
#define EDQUOT          122     /* Quota exceeded */

#define ENOMEDIUM       123     /* No medium found */
#define EMEDIUMTYPE     124     /* Wrong medium type */
#define ECANCELED       125     /* Operation Canceled */
#define ENOKEY          126     /* Required key not available */
#define EKEYEXPIRED     127     /* Key has expired */
#define EKEYREVOKED     128     /* Key has been revoked */
#define EKEYREJECTED    129     /* Key was rejected by service */

/* for robust mutexes */
#define EOWNERDEAD      130     /* Owner died */
#define ENOTRECOVERABLE 131     /* State not recoverable */

#define ERFKILL         132     /* Operation not possible due to RF-kill */

#define EHWPOISON       133     /* Memory page has hardware error */

#endif

#define FILE	void
#define stdin	NULL
#define stdout	NULL
#define stderr	NULL

#define fprintf(fp, fmt, ...)		printf(fmt, __VA_ARGS__)
#define fflush(fp) ((void)0)

#include <gsi/libsys/host_vtable.h>

extern const struct gsi_host_vtable *gsi_host_ops;

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wshadow"

/*
 * io
 */

static int __attribute__((format(printf, 1, 2))) printf(const char *format, ...)
{
	int ret;
	va_list ap;

	va_start(ap, format);
	ret = gsi_host_ops->vprintf(format, ap);
	va_end(ap);

	return ret;
}

static int __attribute__((format(printf, 3, 4))) snprintf(char *str, size_t size, const char *format, ...)
{
	int ret;
	va_list ap;

	va_start(ap, format);
	ret = gsi_host_ops->vsnprintf(str, size, format, ap);
	va_end(ap);

	return ret;
}

static int vprintf(const char *format, va_list ap)
{
	return gsi_host_ops->vprintf(format, ap);
}

static int vsnprintf(char *str, size_t size, const char *format, va_list ap)
{
	return gsi_host_ops->vsnprintf(str, size, format, ap);
}

/*
 * exit
 */
static __attribute__((noreturn)) void exit(int exit_status)
{
	gsi_host_ops->exit(exit_status);
}

static __attribute__((noreturn)) void abort(void)
{
	gsi_host_ops->abort();
}

#define PROT_READ	0x1		/* Page can be read.  */
#define PROT_WRITE	0x2		/* Page can be written.  */
#define PROT_EXEC	0x4		/* Page can be executed.  */
#define PROT_NONE	0x0		/* Page can not be accessed.  */

static int mprotect(void *addr, size_t len, int prot)
{
	return gsi_host_ops->mprotect(addr, len, prot);
}

#pragma GCC diagnostic pop

// #else
// #error "Unsupported product"

/*
 * other
 */

#define ffsll __builtin_ffsll
extern void *memcpy(void *restrict dest, const void *restrict src, size_t n);
#define memcpy __builtin_memcpy
extern void *memset(void *dest, int c, size_t n);
#define memset __builtin_memset
extern void *memchr(const void *src, int c, size_t n);
#define memchr __builtin_memchr
extern void *memrchr(const void *src, int c, size_t n);
#define memrchr __builtin_memrchr
extern int memcmp(const void *restrict s1, const void *restrict s2, size_t n);
#define memcmp __builtin_memcmp
extern int abs(int i);
#define abs __builtin_abs

static inline int __isspace(int _c)
{
	return _c == ' ' || (unsigned)_c - '\t' < 5;
}
#define isspace(c) __isspace(c)

extern size_t strlen(const char *s);
#define strlen __builtin_strlen
extern char *strchrnul(const char *s, int c);
#define strchrnul __builtin_strchrnul
extern char *strchr(const char *s, int c);
#define strchr __builtin_strchr
extern char *strrchr(const char *s, int c);
#define strrchr __builtin_strrchr
extern char *strcpy(char *dest, const char *src);
#define strcpy __builtin_strcpy
extern char *strncpy(char *dest, const char *src, size_t n);
#define strncpy __builtin_strncpy
extern int strcmp(const char *s1, const char *s2);
#define strcmp __builtin_strcmp
extern int strncmp(const char *s1, const char *s2, size_t n);
#define strncmp __builtin_strncmp

extern unsigned long int strtoul(const char *nptr, char **endptr, int base);
extern long int strtol(const char *nptr, char **endptr, int base);

#endif /* !hw */

#endif /* APC_LIBC_H */
